package dybd

import (
	"fmt"
	"gd_service/apis"
	"gd_service/common.in/cache"
	"gd_service/common.in/config"
	"gd_service/consts"
	"gd_service/errors"
	"gd_service/thirdparty"
	dutils "gd_service/utils"
	"strconv"
	"time"

	"github.com/astaxie/beego/orm"

	"gd_service/common.in/utils"
	"go.uber.org/zap"

	"github.com/tidwall/gjson"
)


func getDybdToken(tryAgain bool, httpTimeout int,appKey string,redisKey string) (string, error) {
	token := ""
	if !tryAgain {
		token, _ = cache.Redis.Get(redisKey)
		if token != "" {
			return token, nil
		}
	}

	resp, err := thirdparty.DybdGetToken(appKey,httpTimeout)
	if err != nil {
		l.Error("thridPart",
			zap.String("call", "DybdGetToken"),
			zap.String("args", ""),
			zap.String("error", err.Error()))
		return "", err
	}

	code := gjson.GetBytes(resp, "code").String()
	if code != "200" {
		// 失败的状态
		msg := gjson.GetBytes(resp, "message").String()
		return string(resp), fmt.Errorf("%s",msg)
	}


	token = gjson.GetBytes(resp, "data").String()
	_, _ = cache.Redis.SetEx(redisKey, 23*60*60, token)

	// 写入请求日志
	return token, nil
}


func GetDybdX001Data(rdata string, respMap map[string]string) {
	arr := gjson.Parse(rdata).Array()
	for _, v := range arr {
		respMap["birth_date"] = v.Get("csrq").String()   // 出生日期
		respMap["gender"] = v.Get("xb").String()   // 性别
		respMap["major"] = v.Get("xkzy").String()   // 学科专业
		respMap["degree_name"] = v.Get("xwlb").String()   // 获学位名称
		respMap["award_name"] = v.Get( "xwsydw").String()   // 学位授予单位
		respMap["name"] = v.Get( "xm").String()  //  姓名
		respMap["award_date"] = v.Get( "hxwrq").String()  // 获学位日期
	}
}

func getDybdX001Key(req *apis.ThirdpartRequest) string {
	key := consts.DybdX001Pre + req.Params[consts.DyBdX001P1] + req.Params[consts.DyBdX001P2]
	return key
}

func paramDybdX001Data(ret string) (reply string, err error) {
	code := gjson.Get(ret, "code").Int()
	if code == 200 {
		result := gjson.Get(ret, "data").String()
		arr := gjson.Parse(result).Array()
		if len(arr) == 0 {
			return result, errors.NoRecord
		} else {
			return result, nil
		}
	}else if code == 401{
		return "" ,errors.UserTokenExpire
	} else {
		return "", errors.VendorError
	}
}

func DybdX001Local(req *apis.ThirdpartRequest) (reply string, err error) {
	reply, err = dutils.GetDataByRedis(getDybdX001Key(req), int64(req.Req.ReuseTime), paramDybdX001Data)
	return reply, err
}

func StoreX001DataToMysql(data map[string]string,resData string) {
	x001Data := apis.DybdX001Data{}
	name := data[consts.DyBdX001P2]
	degreeId := data[consts.DyBdX001P1]

	o := orm.NewOrm()
	err := o.Raw("select * from t_gd_dybd_x001_data where degree_id = ? and name=?",degreeId,name).QueryRow(&x001Data)
	if err == nil{
		return
	}

	x001Data.Name = name
	x001Data.DegreeId = degreeId
	x001Data.Data = resData
	x001Data.Timestamp = time.Now().Unix()
	o.Insert(&x001Data)
}

func DybdX001(req *apis.ThirdpartRequest) (reply string, err error) {
	//req.LReq.Search = req.Params[consts.DyBdX001P1]
	//owner := req.Params["owner"]

	redisKey := "dybd:xwtoken:"
	token,err := getDybdToken(req.TryAgain,req.ProviderInfo.Timeout,config.Conf.ThirdPart.DybdXwAppkey,redisKey)
	if err != nil{
		l.Error("thridPart",
			zap.String("call", "getDybdToken"),
			zap.String("args",utils.MarshalJsonString(req.Params)),
			zap.String("error", err.Error()))
		req.LReq.ResponseParams = err.Error()
		dutils.SetAccessLogReqCodeMsgState(req.LReq, 1101, "gd_1101", err.Error(), false)
		return "",errors.VendorError
	}

	ret, err := thirdparty.DyBdClientPost(req.ProviderInfo.ThirdpartHost+"/"+req.ProviderInfo.ThirdpartApiRouter, req.Params, req.ProviderInfo.Timeout,token)
	if err != nil {
		l.Error("func",
			zap.String("call", "DyBdClientPost"),
			zap.String("args", utils.MarshalJsonString(req)),
			zap.String("error", err.Error()))
		req.LReq.ResponseParams = err.Error()
		dutils.SetAccessLogReqCodeMsgState(req.LReq, 1101, "gd_1101", "三方错误,请求失败", false)
		return "", errors.VendorError
	} else {
		req.LReq.ResponseParams = string(ret)
		reply, err = paramDybdX001Data(string(ret))
		code := gjson.GetBytes(ret, "code").String()
		message := gjson.GetBytes(ret, "message").String()
		if err == nil {
			dutils.SetAccessLogReqCodeMsgState(req.LReq, 0, code, message, true)
			dutils.StoreDataToRedis(getDybdX001Key(req), string(ret))
			//go StoreX001DataToMysql(req.Params,reply)
		} else if err == errors.NoRecord {
			dutils.SetAccessLogReqCodeMsgState(req.LReq, 1100, "gd_1100", message, false)
			dutils.StoreDataToRedis(getDybdX001Key(req), string(ret))
		} else if err == errors.UserTokenExpire{
			if !req.TryAgain {
				req.TryAgain = true
				return DybdX001(req)
			}
			err = errors.VendorError
			dutils.SetAccessLogReqCodeMsgState(req.LReq, 1101, code, message, false)
		}else {
			code := gjson.GetBytes(ret, "code").Int()
			l.Error("func",
				zap.String("call", "DyBdClientPost"),
				zap.String("args", utils.MarshalJsonString(req.Params)),
				zap.String("error", err.Error()))
			dutils.SetAccessLogReqCodeMsgState(req.LReq, int(code), strconv.Itoa(int(code)), message, false)
		}

		return reply, err

	}
}
