// Copyright 2019 autocareai.com. All rights reserved.
// Use of this source code is governed by autocareai.com.

package zr

import (
	"gd_service/apis"
	"gd_service/common.in/utils"
	"gd_service/consts"
	"gd_service/errors"
	"gd_service/thirdparty"
	dutils "gd_service/utils"
	"go.uber.org/zap"
	"strings"

	"github.com/tidwall/gjson"
)


func getZrNationalKey(req *apis.ThirdpartRequest) string {
	key := consts.ZRNationalTwoElementVerifyPre + req.Params["plateNum"] +  req.Params["plateColor"] + req.Params["name"]
	return key
}


func getZrNationalPlateNoKey(req *apis.ThirdpartRequest) string {
	key := consts.ZRNationalTwoElementVerifyPre + req.Params["plateNum"] +  req.Params["plateColor"]
	return key
}

func paramZrNationalPlateNoData(ret string) (reply string, err error){
	return ret,nil
}


func paramZrNationalData(ret string) (reply string, err error) {
	code := gjson.Get(ret, "code").String()
	result := gjson.Get(ret, "data.checkResult").String()
	if code == "0" {
		if strings.Contains(result,"2"){
			reply = "1"
		}else if strings.Contains(result,"4") {
			reply = "-1"
		}else if result == "3"{
			// 若核验结果只有3（代办人），代表车辆为企业车辆，则为不一致
			reply = "-1"
		} else if strings.Contains(result,"5"){
			// 查无 不计费
			return "",errors.NoRecord
		}else {
			// 其他计费
			return result,errors.NoRecord
		}
		return reply,nil
	} else if code == "10020"{
		return "", errors.NoRecord
	}else if code == "10003"{
		return "",errors.UserTokenExpire
	}

	return "", errors.VendorError
}

func TwoElementVerifyNationalLocal(req *apis.ThirdpartRequest) (reply string, err error) {
	name := req.Params["name"]
	localOwner, err := dutils.GetDataByRedis(getZrNationalPlateNoKey(req), int64(req.Req.ReuseTime), paramZrNationalPlateNoData)
	if err == nil {
		if localOwner == name{
			return "1",nil
		}else {
			return "-1",nil
		}
	}

	reply, err = dutils.GetDataByRedis(getZrNationalKey(req), int64(req.Req.ReuseTime), paramZrNationalData)
	return reply, err
}

func TwoElementVerifyNational(req *apis.ThirdpartRequest) (reply string, err error) {
	plateNo := req.Params["plateNum"]
	name := req.Params["name"]
	plateType := req.Params["plateColor"]
	color, exist :=  utils.ConvertPlateType(plateType)
	if !exist{
		return "",errors.ArgsError
	}
	req.Params["plateColor"] = color

	if plateNo == "" || name == "" {
		return "",errors.ArgsError
	}


	req.LReq.Search = plateNo
	token,err := getZrToken(req.ProviderInfo.ThirdpartHost,req.TryAgain,req.ProviderInfo.Timeout)
	if err != nil{
		l.Error("thridPart",
			zap.String("call", "getZrToken"),
			zap.String("args",utils.MarshalJsonString(req.Params)),
			zap.String("error", err.Error()))
		req.LReq.ResponseParams = err.Error()
		dutils.SetAccessLogReqCodeMsgState(req.LReq, 1101, "gd_1101", err.Error(), false)
		return "",errors.VendorError
	}

	ret, err := thirdparty.ZrHttpClient(req.ProviderInfo.ThirdpartHost+"/"+req.ProviderInfo.ThirdpartApiRouter, token,req.Params,req.ProviderInfo.Timeout)
	if err != nil {
		l.Error("func",
			zap.String("call", "ZrHttpClient"),
			zap.String("args", utils.MarshalJsonString(req.Params)),
			zap.String("error", err.Error()))
		req.LReq.ResponseParams = err.Error()
		dutils.SetAccessLogReqCodeMsgState(req.LReq, 1101, "gd_1101", "三方错误,请求失败", false)
		return "", errors.VendorError
	} else {
		req.LReq.ResponseParams = string(ret)
		reply ,err = paramZrNationalData(string(ret))
		message := gjson.GetBytes(ret, "msg").String()
		code := gjson.GetBytes(ret, "code").String()
		if err == nil {
			if reply == "-1"{
				dutils.StoreDataToRedis(getZrNationalKey(req), string(ret))
			}else {
				// 一致
				dutils.StoreDataToRedis(getZrNationalPlateNoKey(req), req.Params["name"])
			}
			dutils.SetAccessLogReqCodeMsgState(req.LReq, 0, code, message, true)

		} else if err == errors.NoRecord {
			if reply  != ""{
				// 计费
				dutils.SetAccessLogReqCodeMsgState(req.LReq, 0, code, message, true)
			}else {
				dutils.SetAccessLogReqCodeMsgState(req.LReq, 1100, code, message, false)
			}

			dutils.StoreDataToRedis(getZrNationalKey(req), string(ret))
		}else if err == errors.UserTokenExpire{
			req.Times = req.Times + 1
			if req.Times >= 3 {
				return "", errors.VendorError
			}
			req.TryAgain = true
			return TwoElementVerifyNational(req)
		} else {
			l.Error("func",
				zap.String("call", "ZrHttpClient"),
				zap.String("args", utils.MarshalJsonString(req.Params)),
				zap.String("error", err.Error()))
			//codeInt,_ := strconv.Atoi(code)
			dutils.SetAccessLogReqCodeMsgState(req.LReq, 1101, code, message, false)
		}
		return reply, err
	}
}

